// I2C and SPI0 OLED init

static uint8 CODE initData[25] = 
{
    0xAE,       // display off
    0xD5,0x80,  // set display clock divide ratio/oscillator frequency
    0xA8,0x3F,  // set multiplex ratio(1 to 64)
    0xD3,0x00,
    0x40,
    0x8D,0x14,  // inclusion of an internal voltage converter
    0x20,0x00,  // Set Memory Addressing Mode 00,Horizontal Addressing Mode;01,Vertical Addressing Mode;10,Page Addressing Mode (RESET,0);11,Invalid
    0xA1,       // set segment re-map 0 to 127
    0xC8,       // set COM output scan direction
    0xDA,0x12,  // set COM pins
    0x81,0xCF,  // set contrast control register
    0xD9,0xF1,  // set pre-charge period
    0xDB,0x40,  // set vcomh
    0xA4,       // 0xA4 output follows RAM content; 0xA output ignores RAM content
    0xA6,       // 0xA6 normal / reverse
    0xAF        // display on
};

static uint8 CODE initDataI2cPreamble[8] =
{
	0x78,0x00,
	0x21,0x00,0x7F,
	0x22,0x00,0x07
};

static uint8 CODE initDataSpiPreamble[6] =
{
	0x21,0x00,0x7F,
	0x22,0x00,0x07
};

static uint8 XDATA rxData[0] = {0};
static uint8 XDATA txData[1024] = {0};

// 4-wire SPI0
#define RES0low     setDigitalOutput(1,0)   // P0_1 to RES
#define RES0high    setDigitalOutput(1,1)

#define DC0low      setDigitalOutput(2,0)   // P0_2 to D/C (MISO)
#define DC0high     setDigitalOutput(2,1)
                                            // P0_3 to DIN (MOSI)
#define CS0low      setDigitalOutput(4,0)   // P0_4 to CS0 (!CC)
#define CS0high     setDigitalOutput(4,1)
                                            // P0_5 to CLK
// 4-wire SPI1
#define RES1low     setDigitalOutput(13,0)  // P1_3 to RES
#define RES1high    setDigitalOutput(13,1)

#define DC1low      setDigitalOutput(17,0)  // P1_7 to D/C (MISO)
#define DC1high     setDigitalOutput(17,1)
                                            // P1_6 to DIN (MOSI)
#define CS1low      setDigitalOutput(14,0)  // P1_4 to CS1 (!CC)
#define CS1high     setDigitalOutput(14,1)
                                            // P1_5 to CLK

void i2cInit(void)
{
    i2cPinScl = 10;         // P1_0 to SLC
    i2cPinSda = 11;         // P1_1 to SDA
    i2cSetFrequency(400);   // kHz
    i2cSetTimeout(10);      // ms
}

void spiInit(uint8 spi_number)
{
    if (spi_number == 0)
	{
		spi0MasterInit();
		spi0MasterSetFrequency(3000000);
		spi0MasterSetClockPolarity(SPI_POLARITY_IDLE_LOW);  // Sets the clock polarity CPOL = 0 (SPI_POLARITY_IDLE_HIGH)
		spi0MasterSetClockPhase(SPI_PHASE_EDGE_LEADING);    // Sets the clock phase CPHA = 0 (SPI_PHASE_EDGE_TRAILING)
		spi0MasterSetBitOrder(SPI_BIT_ORDER_MSB_FIRST);     // The most-significant bit is transmitted first
	}
	else
	{
		spi1MasterInit();
		spi1MasterSetFrequency(3000000);
		spi1MasterSetClockPolarity(SPI_POLARITY_IDLE_LOW);
		spi1MasterSetClockPhase(SPI_PHASE_EDGE_LEADING);
		spi1MasterSetBitOrder(SPI_BIT_ORDER_MSB_FIRST);
	}
}

void oledI2cInit(void)
{
    BIT nack;
    uint8 i;

    for (i = 0; i < 25; i++) 
    {
        i2cStart();
        i2cWriteByte(0x78);
        i2cWriteByte(0x00);
        nack = i2cWriteByte(initData[i]);
        if (i2cTimeoutOccurred)
        {
            delayMs(50);
            i2cTimeoutOccurred = 0;
        }
        else if (nack)
            LED_RED(1);
        i2cStop();
    }
}

void oledSpiInit(uint8 spi_number)
{
    uint8 i;
    
	if (spi_number == 0)
	{
		RES0low;
		delayMicroseconds(3);
		RES0high;
		DC0low;
		CS0low;
		for (i = 0; i < 25; i++) 
		{
			spi0MasterSendByte(initData[i]);
			delayMs(1);
		}
		CS0high;
		DC0high;
	}
	else
	{
		RES1low;
		delayMicroseconds(3);
		RES1high;
		DC1low;
		CS1low;
		for (i = 0; i < 25; i++) 
		{
			spi1MasterSendByte(initData[i]);
			delayMs(1);
		}
		CS1high;
		DC1high;
	}
}

void oledI2cFramePreamble(void)
{
    uint8 i;
    
    i2cSetFrequency(400);
    
    i2cStart();
    for (i = 0; i < 8; i++) 
        i2cWriteByte(initDataI2cPreamble[i]);
    i2cStop();
}

void oledSpiFramePreamble(uint8 spi_number)
{
    uint8 i;
    
	if (spi_number == 0)
	{
		DC0low;
		CS0low;
		for (i = 0; i < 6; i++) 
		{
			spi0MasterSendByte(initDataSpiPreamble[i]);
			delayMicroseconds(3);
		}
		CS0high;
	}
	else
	{
		DC1low;
		CS1low;
		for (i = 0; i < 6; i++) 
		{
			spi1MasterSendByte(initDataSpiPreamble[i]);
			delayMicroseconds(3);
		}
		CS1high;
	}
}

void oledInit(uint8 i2c_on, uint8 spi_on, uint8 spi_number)
{
    if (i2c_on)
    {
        i2cInit();
        oledI2cInit();
    }
	
    if (spi_on)
    {
        spiInit(spi_number);
        delayMicroseconds(100);
        oledSpiInit(spi_number);
    }
}

void oledClear(void)
{
	uint16 j;
	
	for (j = 0; j < 1024; j++)
        txData[j] = 0x00;
}

uint16 oledLetter(int16 character, uint16 position, BIT upsideDown) // number use ASCII code
{
	uint8 col;
	uint8 reverse;
	
	for (col = 0; col < 6; col++)
	{
		if (upsideDown)
		{
			reverse = myFont6[character - 0x20][col];
			reverse = (reverse & 0x55) <<  1 | (reverse & 0xAA) >>  1;
			reverse = (reverse & 0x33) <<  2 | (reverse & 0xCC) >>  2;
			reverse = (reverse & 0x0F) <<  4 | (reverse & 0xF0) >>  4;
			txData[position] = reverse;
			position--;
		}
		else
		{
			txData[position] = myFont6[character - 0x20][col];
			position++;
		}
	}
	return position;
}

void oledRow(uint8 reportLength, uint8 row, BIT upsideDown)
{
	uint8 k;
	uint16 position;
	uint8 letter;
	
	if (row >= 1 && row <= 8)
		position = upsideDown ? (1023 - 128 * (row - 1)) : (128 * (row - 1));
	else
		position = upsideDown ? 1023 : 0;
	
	for (k = 0; k < reportLength; k++)
	{
		letter = (uint8)report[k];

		if (letter >= 0x20 && letter != 0xD0) // for special cyrillic symbols
			position = oledLetter(letter, position, upsideDown);
	}
}
/*
void oledTextBlock(uint8 reportLength, uint16 positionStart, BIT upsideDown)
{
	uint8 k;
	uint16 position;
	uint8 letter;
	
	if (positionStart <= 1023)
		position = upsideDown ? (1023 - positionStart) : positionStart;
	
	for (k = 0; k < reportLength; k++)
	{
		letter = (uint8)report[k];

		if (letter >= 0x20 && letter != 0xD0) // for special cyrillic symbols
			position = oledLetter(letter, position, upsideDown);
		
		if ((position + 2) % 128 == 0) // end of the row
			position += 2;
	}
}
*/
void oledShow(uint8 i2c_on, uint8 spi_on, uint8 spi_number)
{
	BIT nack;
    uint8 k;
    uint8 i;
	
	if (i2c_on)
	{
		oledI2cFramePreamble();
		
		for (k = 0; k < 64; k++) {
			i2cStart();
			i2cWriteByte(0x78);
			i2cWriteByte(0x40);
			for (i = 0; i < 16; i++) 
			{
				nack = i2cWriteByte(txData[16 * k + i]);
				if (i2cTimeoutOccurred)
				{
					delayMs(50);
					i2cTimeoutOccurred = 0;
				}
				else if (nack)
					LED_RED(1);
			}
			i2cStop();
		}
	}

	if (spi_on)
	{
		oledSpiFramePreamble(spi_number);
		
		if (spi_number == 0)
		{
			DC0high;
			CS0low;
			spi0MasterTransfer(txData, rxData, 1024);
			delayMs(6);
			CS0high;
		}
		else
		{
			DC1high;
			CS1low;
			spi1MasterTransfer(txData, rxData, 1024);
			delayMs(6);
			CS1high;
		}
	}
}