/** Dependencies **************************************************************/
#include <wixel.h>
#include <usb.h>
#include <usb_com.h>
#include <radio_queue.h>

//int32 CODE param_radio_channel = 128;//162; 2439,8428 2449,5804

/** xTea */
#include <xtea.h>
#include <stdio.h>

/** OLED */
#include <spi0_master.h>
#include <spi1_master.h>
#include <i2c.h>

#include "font6.h"

uint8 XDATA report[21];

#include "oled.h"

// OLED params
uint8 CODE param_i2c_on = 1;
uint8 CODE param_spi_on = 0;
uint8 CODE param_spi_number = 0;
uint8 CODE param_upside_down = 1; // Screen upside down
//int32 CODE param_height = 64;

/** Parameters ****************************************************************/

uint8 CODE param_report_period_ms = 20;

static uint8 CODE buffer[8] = {0x66,0x66,0x65,0x65,0x66,0x66,0x65,0x65};
uint8 XDATA key[16];
uint8 XDATA i;

uint16 XDATA number_low = 0;
uint16 XDATA number_high = 0;

/** Functions *****************************************************************/
void updateLeds(void)
{
    usbShowStatusWithGreenLed();
    LED_YELLOW(0);
    LED_RED(0);
}

void putchar(char c)
{
    usbComTxSendByte(c);
}

// type -> MBEDTLS_XTEA_ENCRYPT 1  MBEDTLS_XTEA_DECRYPT 0
uint8 * encrypt(uint8 * get, int type)
{
	uint8 XDATA input[8], output[8];
	
	mbedtls_xtea_context * enc = NULL;
	sprintf(input, get);
	
	mbedtls_xtea_init(enc);
	mbedtls_xtea_setup(enc, key);
	mbedtls_xtea_crypt_ecb(enc, type, input, output);
	mbedtls_xtea_free(enc);
	
	return output;
}

uint16 frequencyFromChannel(uint8 channel)
{
	uint32 XDATA freq = 34700;
	uint16 XDATA frequency;
	
	freq += 2864 * (uint32)channel;
	
	frequency = (uint16)(freq / 1000);
	
	if (freq % 1000 >= 500)
		frequency++;
	
	return frequency;
}

// This function should be called regularly.
void dataToRadioService(void)
{
    static uint16 lastTx = 0;
    uint8 XDATA * txPacket;
	uint8 XDATA reportLength;
	
    // Check to see if it is time to send a report and
    // if there is a radio TX buffer available.
    if ((uint16)(getMs() - lastTx) >= param_report_period_ms && (txPacket = radioQueueTxCurrentPacket()))
    {
		uint8 routput[8];
        
        lastTx = getMs();

        // Byte 0 is the length.
        txPacket[0] = 12;

        // Bytes 1-4 are the serial number.
        txPacket[1] = serialNumber[0];
        txPacket[2] = serialNumber[1];
        txPacket[3] = serialNumber[2];
        txPacket[4] = serialNumber[3];
		
		sprintf(routput, encrypt(buffer, 1));
		
		for (i = 0; i < 8; i++)
			txPacket[i + 5] = routput[i];
		/*
		printf("|");
		//printf(buffer);
		for (i = 0; i < 8; i++)
			printf("%c", buffer[i]);
		printf("->");
		printf(txPacket);
		printf("|");
		*/
		
		LED_YELLOW(1);
		
		number_low++;
		
		if (number_low >= 10000)
		{
			number_high++;
			number_low = 0;
		}
		
		oledClear();
		
		if (number_high > 1)
			reportLength = sprintf(report, "Packets:%5d%04d  Tx", number_high, number_low);
		else
			reportLength = sprintf(report, "Packets:     %4d  Tx", number_low);
		oledRow(reportLength, 1, param_upside_down);
		
		reportLength = sprintf(report, "Channel:      %3d", CHANNR);
		oledRow(reportLength, 2, param_upside_down);
		
		//reportLength = sprintf(report, "Frequency: 24%02d.%d MHz", frequencyFromChannel(CHANNR) / 10, frequencyFromChannel(CHANNR) % 10);
		reportLength = sprintf(report, "Frequency: 2.4%03d GHz", frequencyFromChannel(CHANNR));
		oledRow(reportLength, 3, param_upside_down);
		
        radioQueueTxSendPacket();
    }
}
void freeBuf(uint8 * buf)
{
	for (i = 0; i < 8; i++)
		buf[i] = 0;
}

void main(void)
{
	systemInit();
    usbInit();
    radioQueueInit();
	
	oledInit(param_i2c_on, param_spi_on, param_spi_number);
	
	sprintf(key, "fffffffffffffff0");
	//sprintf(buffer, "ffeeffee");
	
	// Pull-up
	setDigitalInput(0, 1);  // Pause
	
    while(1)
    {
        updateLeds();
        boardService();
        //usbComService();
		if (isPinHigh(0))   // Pause P0_0
			dataToRadioService();
	
		oledShow(param_i2c_on, param_spi_on, param_spi_number);
    }
}